"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.find_by_prop = exports.filter_and_assert_type = exports.toArrayLiteralString = exports.maximum_by = exports.minimum_by = exports.mutating_delete_at = exports.exclude = exports.zip = exports.zipWith = exports.from_n_to_m = exports.from_0_to_n = exports.from_1_to_n = void 0;
function from_1_to_n(n) {
    return Array.from({ length: n }, (v, i) => i + 1);
}
exports.from_1_to_n = from_1_to_n;
function from_0_to_n(n) {
    return Array.from({ length: n + 1 }, (v, i) => i);
}
exports.from_0_to_n = from_0_to_n;
function from_n_to_m(n, m) {
    return Array.from({ length: m - n }, (v, i) => i + n);
}
exports.from_n_to_m = from_n_to_m;
function zipWith(fn, arr1, arr2) {
    return arr1.map((v, i) => {
        return fn(v, arr2[i]);
    });
}
exports.zipWith = zipWith;
function zip(arr1, arr2) {
    return arr1.map((v, i) => {
        return [v, arr2[i]];
    });
}
exports.zip = zip;
function exclude(arr, exlusionCriteria) {
    return arr.filter((elem) => !exlusionCriteria(elem));
}
exports.exclude = exclude;
function mutating_delete_at(array, index) {
    array.splice(index, 1);
    return array;
}
exports.mutating_delete_at = mutating_delete_at;
function minimum_by(array, sortFn) {
    if (array.length == 0)
        throw new Error("minimum_by applied to empty array");
    return array.sort(sortFn)[0];
}
exports.minimum_by = minimum_by;
function maximum_by(array, sortFn) {
    if (array.length == 0)
        throw new Error("maximum_by applied to empty array");
    return array.sort(sortFn)[array.length - 1];
}
exports.maximum_by = maximum_by;
function toArrayLiteralString(array) {
    if (array.length == 0)
        return "[]";
    if (typeof array[0] == "number") {
        return `[${array.join(", ")}]`;
    }
    else {
        return `["${array.join('", "')}"]`;
    }
}
exports.toArrayLiteralString = toArrayLiteralString;
/**
 * Given a
 * @param type_guard , apparently now called "type predicate", a (ElemT => boolean) function with special return type annotation
 *                   that can assert the type of an element in a way the compiler respects
 *                   (see <https://www.typescriptlang.org/docs/handbook/2/narrowing.html#using-type-predicates>)
 * this function (returns a function that) filters an array to just those that pass the type guard,
 * and makes TS believe the fact that the resulting array is of the asserted type.
 * NB: if the type guard is incomplete/incorrect, the resulting array may not actually contain only elements of the asserted type.
 * TODO: maybe adapt API from
 *  filter_and_assert_type(type_guard)(array) to
 *  filter_and_assert_type(array, type_guard)
 * Also, what would it take to allow
 *  array.filter_and_assert_type(type_guard) ?
 */
function filter_and_assert_type(type_guard) {
    return (elems) => (elems
        .filter((elem) => type_guard(elem))
        .map((elem) => {
        if (type_guard(elem))
            return elem;
        else
            throw new Error("BUG in (usage of?) filter_and_assert_type.");
    }));
}
exports.filter_and_assert_type = filter_and_assert_type;
/**
 * Finds an element from the array by the value of property `prop`.
 */
function find_by_prop(array, prop, value) {
    return array.find(x => x[prop] === value);
}
exports.find_by_prop = find_by_prop;
